package lista_utenti

import (
	"encoding/json"
	"os"

	"git.phc.dm.unipi.it/phc/website/auth"
	"git.phc.dm.unipi.it/phc/website/model"
	"git.phc.dm.unipi.it/phc/website/util"
)

type Service interface {
	GetUtenti() ([]*model.ListUser, error)
}

// New crea un nuovo servizio che lista gli utenti utilizzando AuthService se config è ":auth:" oppure un file json
func New(authService auth.Service, config string) (Service, error) {
	if config == ":auth:" {
		return newAuthListaUtenti(authService)
	}

	return newJsonListaUtenti(config)
}

func loadMacchinisti() (util.Set[string], error) {
	f, err := os.Open("macchinisti.json")
	if err != nil {
		return nil, err
	}

	macchinisti := []string{}

	if err := json.NewDecoder(f).Decode(&macchinisti); err != nil {
		return nil, err
	}

	return util.NewSet(macchinisti...), nil
}

func mergeMacchinisti(users []*model.ListUser, macchinisti util.Set[string]) {
	for _, user := range users {
		if _, found := macchinisti[user.Uid]; found {
			user.Tags = append(user.Tags, "macchinista")
		}
	}
}
