package model

type ListUser struct {
	Uid     string   `json:"uid"`
	Nome    string   `json:"nome"`
	Cognome string   `json:"cognome"`
	Tags    []string `json:"tags,omitempty"`
}

// User represents a user returned from AuthenticatorService
type User struct {
	Username string `json:"username"`
	Name     string `json:"name"`
	Surname  string `json:"surname"`
	// FullName is a separate field from Name and Surname because for example
	// ldap stores them all as separate fields.
	FullName string `json:"fullName"`
	Email    string `json:"email"`
}

// WithDefaultFullName is a utility that returns a copy of the given user with the full name set to the concatenation of the name and surname of the user.
func (u User) WithDefaultFullName() User {
	return User{
		Username: u.Username,
		Name:     u.Name,
		Surname:  u.Surname,
		Email:    u.Email,

		FullName: u.Username + " " + u.Surname,
	}
}

// Session represents a session returned from AuthenticatorService
type Session struct {
	Username string `json:"username"`
	Token    string `json:"token"`
}
